"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TilesetPackageValidator = void 0;
const path_1 = __importDefault(require("path"));
const defined_1 = require("../base/defined");
const Buffers_1 = require("../base/Buffers");
const Validators_1 = require("./Validators");
const PackageResourceResolver_1 = require("../io/PackageResourceResolver");
const ContentValidationIssues_1 = require("../issues/ContentValidationIssues");
const IoValidationIssue_1 = require("../issues/IoValidationIssue");
const TilesetPackage3tz_1 = require("../packages/TilesetPackage3tz");
const TilesetPackage3dtiles_1 = require("../packages/TilesetPackage3dtiles");
const TilesetPackageFs_1 = require("../packages/TilesetPackageFs");
const ArchiveValidation3tz_1 = require("../packages/ArchiveValidation3tz");
/**
 * An implementation of a validator that validates a `TilesetPackage`.
 *
 * The actual validation function is `validatePackageFile`.
 *
 * This class also implements the `Validator` interface, so that
 * instances of it can be used when the tile content is a
 * package. In this case, the validated type is `string`, where
 * this string is the 'resolvedUri' that points to a file in the
 * local file system.
 *
 * @internal
 */
class TilesetPackageValidator {
    /**
     * Implementation of the `Validator` interface. This validates
     * the given 'resolvedUri' string, assuming that it is a full
     * path to a tileset package in the local file system.
     *
     * @param uri - The (usually relative) URI of the package
     * @param resolvedUri - The resolved URI, which is the full URI
     * of the package in the local file system.
     * @param context - The `ValidationContext`
     * @returns A promise that is fulfilled when the validation is finished
     * and indicates whether the object was valid or not.
     */
    async validateObject(uri, resolvedUri, context) {
        //console.log("TilesetPackageValidator resolvedUri is " + resolvedUri);
        const isContent = true;
        const result = TilesetPackageValidator.validatePackageFileInternal(resolvedUri, isContent, context);
        return result;
    }
    /**
     * Validates the tileset that is contained in the package that is
     * pointed to by the given URI (assuming that it is a file in
     * the local file system).
     *
     * @param uri - The full URI of the package file
     * @param context - The `ValidationContext`
     * @returns A promise that indicates whether the package contained
     * a valid tileset.
     */
    static async validatePackageFile(uri, context) {
        const isContent = false;
        const result = TilesetPackageValidator.validatePackageFileInternal(uri, isContent, context);
        return result;
    }
    /**
     * Validates the tileset that is contained in the package that is
     * pointed to by the given URI (assuming that it is a file in
     * the local file system).
     *
     * @param uri - The full URI of the package file
     * @param isContent - Whether the given package was found as a tile
     * content. If this is the case, then the issues that are found
     * in the package will be summarized in a `CONTENT_VALIDATION_`
     * issue. Otherwise, they will be added directly to the given context.
     * @param context - The `ValidationContext`
     * @returns A promise that indicates whether the package contained
     * a valid tileset.
     */
    static async validatePackageFileInternal(uri, isContent, context) {
        // Create the package from the given URI (i.e. the
        // full package file name). If the package cannot
        // be opened, bail out with an IO_WARNING.
        let tilesetPackage = undefined;
        const extension = path_1.default.extname(uri).toLowerCase();
        if (extension === ".3tz") {
            tilesetPackage = new TilesetPackage3tz_1.TilesetPackage3tz();
        }
        else if (extension === ".3dtiles") {
            tilesetPackage = new TilesetPackage3dtiles_1.TilesetPackage3dtiles();
        }
        else if (extension === "") {
            tilesetPackage = new TilesetPackageFs_1.TilesetPackageFs();
        }
        else {
            const message = `Could not create package from ${uri}: No known file extension. `;
            const issue = IoValidationIssue_1.IoValidationIssues.IO_WARNING(uri, message);
            context.addIssue(issue);
            return true;
        }
        tilesetPackage.open(uri);
        const result = await TilesetPackageValidator.validatePackageInternal(uri, tilesetPackage, isContent, context);
        tilesetPackage.close();
        return result;
    }
    /**
     * Validates the tileset that is contained in the given `TilesetPackage`.
     *
     * The caller is responsible for calling 'open' on the package before
     * passing it to this method, and 'close' after this method returns.
     *
     * @param uri - The full URI of the package file
     * @param tilesetPackage - The `TilesetPackage`
     * @param isContent - Whether the given package was found as a tile
     * content. If this is the case, then the issues that are found
     * in the package will be summarized in a `CONTENT_VALIDATION_`
     * issue. Otherwise, they will be added directly to the given context.
     * @param context - The `ValidationContext`
     * @returns A promise that indicates whether the package contained
     * a valid tileset.
     */
    static async validatePackageInternal(uri, tilesetPackage, isContent, context) {
        // If the package is a 3TZ package, then perform the extended
        // validation of the 3TZ index part, using the ("legacy") 3TZ
        // package validation
        if (tilesetPackage instanceof TilesetPackage3tz_1.TilesetPackage3tz) {
            const tilesetPackage3tz = tilesetPackage;
            const zipIndex = tilesetPackage3tz.getZipIndex();
            if ((0, defined_1.defined)(zipIndex)) {
                try {
                    const indexValid = await ArchiveValidation3tz_1.ArchiveValidation3tz.validateIndex(zipIndex, uri, false);
                    if (!indexValid) {
                        const message = `The 3TZ index is not valid`;
                        const issue = ContentValidationIssues_1.ContentValidationIssues.CONTENT_VALIDATION_ERROR(uri, message);
                        context.addIssue(issue);
                        return false;
                    }
                }
                catch (error) {
                    const message = `Error while validating 3TZ index: ${error}.`;
                    const issue = ContentValidationIssues_1.ContentValidationIssues.CONTENT_VALIDATION_ERROR(uri, message);
                    context.addIssue(issue);
                    return false;
                }
            }
        }
        // Create the `PackageResourceResolver` from the package,
        // and obtain the data for the `tileset.json` file.
        // This has to be present according to the 3TZ specification.
        const packageResourceResolver = new PackageResourceResolver_1.PackageResourceResolver("./", uri, tilesetPackage);
        const tilesetJsonBuffer = await packageResourceResolver.resolveData("tileset.json");
        if (!(0, defined_1.defined)(tilesetJsonBuffer)) {
            const message = `Could not read 'tileset.json' from package ${uri}.`;
            const issue = IoValidationIssue_1.IoValidationIssues.IO_ERROR(uri, message);
            context.addIssue(issue);
            return false;
        }
        const bom = Buffers_1.Buffers.getUnicodeBOMDescription(tilesetJsonBuffer);
        if ((0, defined_1.defined)(bom)) {
            const message = `Unexpected BOM in subtree JSON buffer: ${bom}`;
            const issue = IoValidationIssue_1.IoValidationIssues.IO_ERROR(uri, message);
            context.addIssue(issue);
            return false;
        }
        // Parse the tileset object from the JSON data
        let tileset = undefined;
        try {
            tileset = JSON.parse(tilesetJsonBuffer.toString());
        }
        catch (error) {
            const message = `Could not parse tileset JSON from 'tileset.json' ` +
                `data in package ${uri}.`;
            const issue = IoValidationIssue_1.IoValidationIssues.IO_ERROR(uri, message);
            context.addIssue(issue);
            return false;
        }
        // Open a new context for collecting the issues that
        // are caused by the tileset, and validate the
        // tileset using a default tileset validator.
        const derivedContext = context.deriveFromResourceResolver(packageResourceResolver);
        const tilesetValidator = Validators_1.Validators.createDefaultTilesetValidator();
        const result = await tilesetValidator.validateObject(uri + "/tileset.json", tileset, derivedContext);
        const derivedResult = derivedContext.getResult();
        if (isContent) {
            const issue = ContentValidationIssues_1.ContentValidationIssues.createForContent(uri, derivedResult);
            if (issue) {
                context.addIssue(issue);
            }
        }
        else {
            for (const innerIssue of derivedResult.issues) {
                context.addIssue(innerIssue);
            }
        }
        return result;
    }
}
exports.TilesetPackageValidator = TilesetPackageValidator;
